#
# See the file "LICENSE" for the full license governing this code.
#
require 'pathname'
$LOAD_PATH.unshift(Pathname.new(__FILE__).dirname.parent.parent)
$LOAD_PATH.unshift(Pathname.new(__FILE__).dirname.parent.parent.parent.parent + 'easy_type' + 'lib')
require 'easy_type'
require 'puppet_x/enterprisemodules/iib/commands'
require 'puppet_x/enterprisemodules/iib/settings'
require 'puppet_x/enterprisemodules/iib/broker_global_property'
require 'puppet_x/enterprisemodules/iib/broker_shared_property'
require 'puppet_x/enterprisemodules/iib/broker_keystore_property'
require 'puppet_x/enterprisemodules/iib/broker_info'

Puppet::Type.newtype(:iib_broker) do
  include EasyType
  include Puppet_X::EnterpriseModules::Iib::Commands
  include Puppet_X::EnterpriseModules::Iib::Settings
  extend  Puppet_X::EnterpriseModules::Iib::BrokerInfo

  def self.module_name
    'iib_config'
  end

  desc <<-DOC
This resource allows you to manage broker in an IBM IIB realm.

Here is an example on how you should use this:

    iib_broker { 'IIB107/IB10NODE1':
      ensure => 'present',
      status => 'running',
    }

In this example you are ensuring that broker is running. If the server is
stopped the resource starts broker, if broker is absent then new is created.

Valid managable global node component names:
- userLilPath
- configurationChangeTimeout
- internalConfigurationTimeout
- httpListenerPort
- statisticsMajorInterval
- ldapPrincipal
- ldapCredentials
- ICU converter path
- userExitPath
- activeUserExits
- function level
- security_status
- cachePolicy
- cachePortRange
- integrationRegistryHostname

  DOC

  ensurable

  map_titles_to_attributes([%r{((.*)\/(.*))}, [:name, :instance, :broker]])

  index_regexp = /.*Integration node '(\S+)'(?: .*?queue manager '(\S+?)')? .*?(?:administration URI '(.\S+?)' )?is (\S*)\s*\./

  to_get_raw_resources do
    Puppet.debug 'Getting raw resources for iib_execution_path'
    configuration.keys.collect do |iib_instance|
      Puppet.debug "index of #{iib_instance}"
      information = mqsilist('-a -r', { :instance => iib_instance }).scan(index_regexp)
      information.each_with_index do |element, i|
        # Add instance info
        element << iib_instance
        element << global_properties_output(information[i][0], iib_instance)
        element << (element[3] == 'running' ? shared_properties(information[i][0], iib_instance) : '')
      end

      information.collect do |e|
        Hash[[name,
              :queue_manager,
              :admin_url,
              :status,
              :instance,
              :global_output,
              :shared_properties_output].zip(e)]
      end
    end.flatten
  end

  on_create do
    Puppet.debug "Creating broker #{broker}"
    [:mqsicreatebroker, broker, { :instance => instance }]
  end

  on_modify do
    # Delegate to the properties
  end

  after_modify do
    if self[:status] == :running && provider.status == 'stopped' && self[:auto_restart].to_s == 'true'
      Puppet.debug "Starting broker #{broker} after applying global property."
      mqsistart(broker, { :instance => instance })
    end
  end

  on_destroy do
    if provider.status == 'running'
      Puppet.debug "Stopping broker #{broker} before removal."
      mqsistop(broker, { :instance => instance })
    end
    Puppet.debug "Removing broker #{broker}"
    mqsideletebroker(broker, { :instance => instance })
  end

  # Uncomment to see attribute string used for parsing global attributes.
  # property  :global_output
  # Uncomment to see attributes hash used for separate attributes.
  # property  :shared_properties_output

  parameter :name
  parameter :broker
  parameter :instance
  parameter :auto_restart

  property  :admin_url
  property  :queue_manager

  # Global broker properties
  # Accessible and manageable when the broker is stopped.
  property  :active_user_exits
  property  :cache_policy
  property  :cache_port_range
  property  :configuration_change_timeout
  property  :icu_converter_path
  property  :integration_registry_hostname
  property  :internal_configuration_timeout
  property  :ldap_credentials
  property  :ldap_principal
  property  :security_status
  property  :statistics_major_interval
  property  :user_exit_path
  property  :user_lil_path
  property  :function_level
  #
  # Status needs to be after all global broker properties
  #
  property  :status
  #
  # Shared broker properties
  # Accessible and manageable only when the broker running
  property  :http_listener_allow_ssl_v3
  property  :http_listener_enable_ssl_connector
  property  :http_listener_http_connector_port
  property  :http_listener_https_connector_port
  property  :http_listener_java_debug_port
  property  :http_listener_start_listener
  property  :http_listener_thread_pool_size
  property  :http_listener_trace_level
  property  :http_listener_trace_override_level
  property  :http_listener_trace_override_size
  property  :security_cache_sweep_interval
  property  :security_cache_timeout
  property  :service_federation_scmp_enable_ssl
  property  :service_federation_scmp_enabled
  property  :http_connector_enable_lookups
  property  :https_connector_enable_lookups
  #
  # Keystore properties
  property  :https_connector_keystore_file
  property  :https_connector_keystore_pass
  property  :broker_keystore_file
  property  :broker_truststore_file
  #
  # Currently not supported properties
  #
  # property  :http_listener_trace_size
end
