#
# See the file "LICENSE" for the full license governing this code.
#
require 'pathname'
$LOAD_PATH.unshift(Pathname.new(__FILE__).dirname.parent.parent)
$LOAD_PATH.unshift(Pathname.new(__FILE__).dirname.parent.parent.parent.parent + 'easy_type' + 'lib')
require 'easy_type'
require 'puppet_x/enterprisemodules/iib/ace_check'
require 'puppet_x/enterprisemodules/iib/ace_helpers'
require 'puppet_x/enterprisemodules/iib/commands'
require 'puppet_x/enterprisemodules/iib/node_component'
require 'puppet_x/enterprisemodules/iib/server_array_property'
require 'puppet_x/enterprisemodules/iib/server_info'
require 'puppet_x/enterprisemodules/iib/server_property'
require 'puppet_x/enterprisemodules/iib/settings'

Puppet::Type.newtype(:iib_property) do
  include EasyType
  include Puppet_X::EnterpriseModules::Iib::AceCheck
  include Puppet_X::EnterpriseModules::Iib::AceHelpers
  include Puppet_X::EnterpriseModules::Iib::Commands
  include Puppet_X::EnterpriseModules::Iib::Settings
  extend  Puppet_X::EnterpriseModules::Iib::NodeComponent
  extend  Puppet_X::EnterpriseModules::Iib::ServerInfo

  def self.module_name
    'iib_config'
  end

  desc <<-DOC
This resource allows you to manage brokers and servers in an IBM IIB realm.

This type can be used to construct `mqsichangebroker` or `mqsichangeproperties`
commands for you setup. This type is a fallback for when the property is not
(yet) support on `iib_broker` or `iib_server`. Use this type with care.

Here is an example on how to write manifest for global node component value
changing:

    iib_property {'set_node_cache_policy':
      scope     => 'iib8/NODE1',
      component => 'cachePolicy',
      restart   => true,
      set_value => 'none'
    }


Valid global node component names:
- userLilPath
- configurationChangeTimeout
- internalConfigurationTimeout
- httpListenerPort
- statisticsMajorInterval
- ldapPrincipal
- ldapCredentials
- ICU converter path
- userExitPath
- activeUserExits
- function level
- security_status
- cachePolicy
- cachePortRange
- integrationRegistryHostname

Here is an example on how to write manifest for node component that takes
objectName:

    iib_property {'set_node_cache_policy':
      scope            => 'iib8/NODE1',
      component        => 'webadmins',
      object_name      => 'HTTPConnector',
      object_property  => 'compression',
      restart          => false,
      set_value        => 'on'
    }

Make sure that selected node is running before applying manifest.

Valid node component names that take objectName:
- httplistener
- securitycache
- cachemanager
- webadmin
- crlFile

Here is an example on how to write manifest for server property
changing:

    iib_property {'set_server_accept_count_to_200':
      scope       => 'iib8/NODE1/server_1',
      component   => 'HTTPConnector',
      object_name => 'acceptCount',
      restart     => false,
      set_value   => '200'
    }

  DOC

  map_titles_to_attributes([/(.*)/, [:name]])

  to_get_raw_resources { [] }

  on_prefetch do |resources, provider|
    resources.each do | name, value|
      scope           = value[:scope]
      parsed_scope    = parse_scope(scope)
      scope_type      = parsed_scope[:type]
      instance        = parsed_scope[:instance]
      broker          = parsed_scope[:broker]
      server          = parsed_scope[:server]
      component       = value[:component]
      object_name     = value[:object_name]
      object_property = value[:object_property]

      if ace_used?(instance)
        #
        # Map prefetched names to keys in ace configuration yaml.
        #
        property_yaml_maps_to(component, object_name, object_property)

        current_value     = yaml_value({server: server, broker: broker}, scope_type)
        provider_instance = provider.new(:name => name, :set_value => current_value)
        return resources[name].provider = provider_instance
      end

      parsed_value = case scope_type
      when :broker
        if object_name.nil?
          fetched_value = mqsireportbroker(broker, { :instance => instance })
          report_broker_property_value(fetched_value, component)
        else
          fetched_value = mqsireportproperties("#{broker} -b #{component}" \
                                               " -o #{object_name} -n " \
                                               " #{object_property}",
                                               { :instance => instance })
          report_properties_value(fetched_value)
        end
      when :server
        fetched_value = mqsireportproperties("#{broker} -o #{component} -n " \
                                             "#{object_name} -e #{server}",
                                             { :instance => instance })
        report_properties_value(fetched_value)
      end
      current_value     = parsed_value
      provider_instance = provider.new(:name => name, :set_value => current_value)
      resources[name].provider = provider_instance
    end
  end

  before_modify do
    return if restart.to_s == 'false'
    parsed_scope = parse_scope(scope)
    scope_type   = parsed_scope[:type]
    instance     = parsed_scope[:instance]
    broker       = parsed_scope[:broker]
    server       = parsed_scope[:server]
    case scope_type
    when :server
      Puppet.debug "Stopping server #{server} because restart is specfied."
      mqsistopmsgflow("#{broker} -e #{server}", { :instance => instance })
    when :broker
      Puppet.debug "Stopping broker #{broker} because restart is specfied."
      mqsistop(broker, { :instance => instance })
    else
      fail "scope #{scope} invalid."
    end
  end

  on_modify do
    parsed_scope = parse_scope(scope)
    scope_type   = parsed_scope[:type]
    instance     = parsed_scope[:instance]
    broker       = parsed_scope[:broker]
    server       = parsed_scope[:server]

    if ace_used?(instance)
      #
      # Based on on prefetech data, map the object to yaml keys in ace node file
      #
      property_yaml_maps_to(component, object_name, object_property)

      self.class.write_to_yaml({ broker: broker, server: server },
                               set_value,
                               scope_type)
    end

    case scope_type
    when :server
      #
      # The scope specified is a server. So apply server property
      #
      standard_property = Puppet_X::EnterpriseModules::Iib::ServerProperty
      standard_property.attribute_value(broker,
                                        component,
                                        server,
                                        object_name,
                                        set_value,
                                        instance)
    when :broker
      if object_specfied?
        mqsichangeproperties("#{broker} -b #{component} -o #{object_name} " \
                             "-n #{object_property} -v #{set_value}",
                             { :instance => instance })
      else
        mqsichangebroker("#{broker} #{self.class.component_modifier(component)}" \
                         " #{set_value}",
                         { :instance => instance })
      end
    else
      fail "scope #{scope} invalid with other parameters."
    end
  end

  after_modify do
    return if restart.to_s == 'false'
    parsed_scope = parse_scope(scope)
    scope_type   = parsed_scope[:type]
    instance     = parsed_scope[:instance]
    broker       = parsed_scope[:broker]
    server       = parsed_scope[:server]
    case scope_type
    when :server
      Puppet.debug "Restarting server #{server} because restart is specfied."
      mqsistartmsgflow(server, { :instance => instance })
    when :broker
      Puppet.debug "Restarting broker #{broker} because restart is specfied."
      mqsistart(broker, { :instance => instance })
    else
      fail "scope #{scope} invalid."
    end
  end

  on_destroy do
    fail 'Destroy not valid for this type.'
  end

  parameter :name
  parameter :scope
  parameter :component
  parameter :object_property
  parameter :object_name
  parameter :restart

  property :set_value

  private

  def object_specfied?
    !object_name.nil?
  end

  def parse_scope(scope)
    self.class.parse_scope(scope)
  end

  def self.parse_scope(scope)
    invalid_syntax = 'Invalid scope syntax used'
    split_scope = scope.split('/') { raise invalid_syntax }
    case split_scope.size
    when 3
      return {
        :type => :server,
        :instance => split_scope[0],
        :broker => split_scope[1],
        :server => split_scope[2]
      }
    when 2
      return {
        :type => :broker,
        :instance => split_scope[0],
        :broker => split_scope[1],
        :server => nil
      }
    else
      raise invalid_syntax
    end
  end

  def self.report_properties_value(string)
    value = string.scan(/\n(.*)\n\n.*Successful command completion\./).first
    if value
      value.first
    else
      raise "Invalid combination of properties found. Check IIB message."
    end
  end

  def self.report_broker_property_value(string, component)
    value = string.scan(/#{Regexp.escape(component_regexp_string(component))}\s=\s\'([^\']*)\'/).first
    if value
      value.first
    else
      raise "Invalid combination of properties found. Check IIB message."
    end
  end
end
