#
# See the file "LICENSE" for the full license governing this code.
#

require 'puppet_x/enterprisemodules/iib/array_and_hash_toolkit'
require 'puppet_x/open_source/bottomless_hash'

module Puppet_X
  module EnterpriseModules
    module Iib
      #
      # This is module responsible for handling YAML changes of IBM ACE.
      #
      module AceHelpers
        def self.included(parent)
          parent.extend(AceHelpers)
        end

        #
        # Mapping of iib_property cli names to yaml configuration keys
        #
        PROPERTY_DICTIONARY = {
          ['webadmin', 'HTTPConnector', 'port'] => ['RestAdminListener', 'port'],
          ['crlFile'] => ['BrokerRegistry', 'brokerCRLFileList']
        }.freeze

        #
        # Location of object configuration file
        #
        def yaml_properties_file(resource, type_name)
          path = yaml_file_path(resource, type_name)
          YAML.load_file(path)
        end

        #
        # Path to yaml configuration file for server or broker
        #
        def yaml_file_path(resource, type_name)
          hash_resource = resource.to_hash

          if type_name == 'iib_server' || type_name == 'server'
            "/var/mqsi/components/#{hash_resource[:iib_broker] || hash_resource[:broker]}/servers/#{hash_resource[:iib_server] || hash_resource[:server]}/server.conf.yaml"
          elsif type_name == 'iib_broker' || type_name == 'broker'
            "/var/mqsi/components/#{hash_resource[:iib_broker] || hash_resource[:broker]}/node.conf.yaml"
          else
            "/var/mqsi/components/#{hash_resource[:iib_broker] || hash_resource[:broker]}/node.conf.yaml"
          end
        end

        #
        # Mapping for key names in yaml file
        #
        def yaml_maps_to(*yaml_mapping_array)
          self.yaml_mapping_array(yaml_mapping_array)
        end

        #
        # Mapping to root level key name (optional)
        #
        def yaml_mapping_array(yaml_mapping_array = nil)
          @yaml_mapping_array = yaml_mapping_array || @yaml_mapping_array
        end

        #
        # Reads of yaml hash yaml value
        #
        def yaml_value(raw_resource, type_name)
          return nil unless yaml_mapping_array

          # Depending on type perform different action
          resource_yaml = raw_resource[:global_output] ||
                          raw_resource[:output] ||
                          yaml_properties_file(raw_resource, type_name)

          yaml_lookup(resource_yaml)
        end

        #
        # Lookup in yaml. Using lookup module method ./lookup.rb
        #
        def yaml_lookup(resource_yaml)
          resource_yaml.search(*yaml_mapping_array)
        end

        #
        # Update the hash with system setting,
        #
        def yaml_object_update(resource_yaml, yaml_mapping_array, value)
          object = resource_yaml.bottomless # Create bottomless hash
          object.set(value, *yaml_mapping_array) # use array_and_hash_toolkit
          object
        end

        #
        # Only for iib_property!
        # Based on on prefetech data, map the object to yaml keys in ace node file
        #
        def property_yaml_maps_to(component, object_name, object_property)
          keys = [component, object_name, object_property]
          yaml_mapping = PROPERTY_DICTIONARY[keys]
          yaml_mapping_array(yaml_mapping)
        end

        #
        # Write to yaml configuratoin server or broker file
        #
        def write_to_yaml(resource, value, type_name)
          return nil unless yaml_mapping_array

          resource_yaml = yaml_properties_file(resource, type_name)

          # Depending on number of nested yaml names perform different write
          resource_yaml = yaml_object_update(resource_yaml, yaml_mapping_array, value)

          file_location = yaml_file_path(resource, type_name)
          File.open(file_location, 'w') { |f| f.write resource_yaml.to_yaml }
          Puppet.notice "The YAML node configuration was changed:\n#{file_location}"
          value
        end
      end
    end
  end
end
